import numpy as np #linear algebra
import pandas as pd #data processing, CSV file
from sklearn.model_selection import train_test_split
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.pipeline import Pipeline
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.preprocessing import StandardScaler
from sklearn.pipeline import FeatureUnion
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import classification_report
from sklearn.model_selection import GridSearchCV
from sklearn.cross_validation import cross_val_score, cross_val_predict
from sklearn import metrics
from sklearn.feature_selection import SelectFromModel
from sklearn.svm import LinearSVC
from sklearn.feature_selection import SelectKBest
from sklearn.metrics import confusion_matrix


# A transformer to select a single column from the data frame to perform additional transformations on. Use on texts columns in the data. 
class TextSelector(BaseEstimator, TransformerMixin):
	def __init__(self, key):
		self.key = key

	def fit(self, X, y=None):
		return self

	def transform(self, X):
		return X[self.key]


# Loads the data
df = pd.read_csv('dataset.csv')

df.dropna(axis=0)
df.set_index('ID', inplace=True)


# Specify the (numeric) features and target columns
text_features = [c for c in df.columns.values if c in ['Text']]
target = 'Label'


# Split the data into training and test sets
X_train, X_test, y_train, y_test = train_test_split(df[text_features], df[target], test_size=0.3, random_state=0)


# Transform the feature columns
text = Pipeline([
	('selector', TextSelector(key='Text')),
	('tfidf', TfidfVectorizer())
	])


# Uncomment for selectKBest algorithm
pipeline = Pipeline([
	('features', text),
	#('feature_selection', SelectKBest(k=10)),
	('classifier', LogisticRegression())
	])

pipeline.fit_transform(X_train, y_train)

# Set of parameters for n-gram
parameters = {
	'features__tfidf__ngram_range':[(1,1), (1,2), (1,3), (1,4), (1,5), (1,6)],
	'features__tfidf__analyzer': ['char', 'word']
}

# Grid search with parameters
LRclassifier = GridSearchCV(estimator=pipeline, param_grid=parameters, cv=10)
LRclassifier.fit(X_train, y_train)

print LRclassifier.best_params_
print LRclassifier.best_estimator_
print "Grid scores: ", LRclassifier.grid_scores_

# Print the 10 best scoring text features
#indices = pipeline.named_steps['feature_selection'].get_support(indices=True)
#available_features = text.named_steps['tfidf'].get_feature_names()
#for index in indices:
#	print available_features[index]

# Print classification report (precision, recall, macro f1-score)
preds = LRclassifier.predict(X_test)
print(classification_report(y_test, preds))

# Confusion matrices
print(confusion_matrix(y_test, preds))



