import numpy as np #linear algebra
import pandas as pd #data processing, CSV file
from sklearn.model_selection import train_test_split
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.pipeline import Pipeline
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.preprocessing import StandardScaler
from sklearn.pipeline import FeatureUnion
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import classification_report
from sklearn.model_selection import GridSearchCV
from sklearn.cross_validation import cross_val_score, cross_val_predict
from sklearn import metrics
from sklearn.feature_selection import SelectKBest
from sklearn.feature_selection import chi2
from sklearn.feature_selection import SelectFromModel
from sklearn.svm import LinearSVC
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.metrics import confusion_matrix
import matplotlib.pyplot as plt


# A transformer to select a single column from the data frame to perform additional transformations on. Use on texts columns in the data. 
class TextSelector(BaseEstimator, TransformerMixin):
	def __init__(self, key):
		self.key = key

	def fit(self, X, y=None):
		return self

	def transform(self, X):
		return X[self.key]

# A transformer to select a single column from the data frame to perform additional transformations on. Use on numeric columns in the data. 
class NumberSelector(BaseEstimator, TransformerMixin):
	def __init__(self, key):
		self.key= key

	def fit(self, X, y=None):
		return self

	def transform(self, X):
		return X[[self.key]]


# Loads the data
df = pd.read_csv('dataset.csv')

# Specify the features and target columns
features = [c for c in df.columns.values if c not in ['Label', 'ID']]
target = 'Label'

# Split the data into training and test sets
X_train, X_test, y_train, y_test = train_test_split(df[features], df[target], test_size=0.3, random_state=0)


# Transform the feature columns
text = Pipeline([
	('selector', TextSelector(key='Text')),
	('tfidf', TfidfVectorizer())
	])

male = Pipeline([
	('selector', NumberSelector(key='Male')),
	('scaler', StandardScaler())
	])

female = Pipeline([
	('selector', NumberSelector(key='Female')),
	('scaler', StandardScaler())
	])

friends = Pipeline([
	('selector', NumberSelector(key='Friends')),
	('scaler', StandardScaler())
	])

followers = Pipeline([
	('selector', NumberSelector(key='Followers')),
	('scaler', StandardScaler())
	])

statuses = Pipeline([
	('selector', NumberSelector(key='Statuses')),
	('scaler', StandardScaler())
	])

favourites = Pipeline([
	('selector', NumberSelector(key='Favourites')),
	('scaler', StandardScaler())
	])

public_lists = Pipeline([
	('selector', NumberSelector(key='Public_lists')),
	('scaler', StandardScaler())
	])

geo_enabled = Pipeline([
	('selector', NumberSelector(key='Geo')),
	('scaler', StandardScaler())
	])

default_profile = Pipeline([
	('selector', NumberSelector(key='Default_profile')),
	('scaler', StandardScaler())
	])

default_image = Pipeline([
	('selector', NumberSelector(key='Default_image')),
	('scaler', StandardScaler())
	])

# Combine the text and numeric features 
feats = FeatureUnion([
	('text', text), 
	('male', male),
	('female', female),
	('friends', friends),
	('followers', followers),
	('statuses', statuses),
	('favourites', favourites),
	('public_lists', public_lists),
	('geo_enabled', geo_enabled),
	('default_profile', default_profile),
	('default_image', default_image)
])

# All features
pipeline = Pipeline([
	('features', feats),
	#('feature_selection', SelectFromModel(LinearSVC(penalty='l2'))),
	('classifier', LogisticRegression())
])

pipeline.fit_transform(X_train, y_train)

# Set of parameters for n-gram
parameters = {
	'features__text__tfidf__ngram_range':[(1,1), (1,2), (1,3), (1,4), (1,5), (1,6)],
	'features__text__tfidf__analyzer': ['word', 'char']
}

# Grid search with parameters
LRclassifier = GridSearchCV(estimator=pipeline, param_grid=parameters, cv=10)
LRclassifier.fit(X_train, y_train)

print LRclassifier.best_params_

# Print classification report (precision, recall, macro f1-score)
preds = LRclassifier.predict(X_test)
print(classification_report(y_test, preds))

# Confusion matrices
cm = confusion_matrix(y_test, preds)
print cm




